#########################
## Section 6
## Logits and quantities of interest
## Stephen Pettigrew
## 3-4-2015
#########################

## First set your working directory
setwd("c:/[your working directory]")

## Now read the votes0410 Stata file
## You'll need the 'foreign' package to be able to read
## Stata files

#install.packages("foreign") # install it if you don't have it

# Load it with either require() or library()
require(foreign)
## or
library(foreign)


## Now load the file
votes <- read.dta("votes0408.dta")


## Look at the dataset
head(votes)
apply(votes, 2, summary)

# incwin: dependent variable. whether incumbent/incumbent's party won the election (1) or not (0)
# open: is the seat open (1) or is there an incumbent running (0)
# freshman: is the incumbent a freshman (1) or not (0)
# incpres: percentage of votes received by incumbent party in the district in last presidential election












###################
## Logit model
###################

## First code a function for the log likelihood

logit.ll <- function(par, outcome, covariates){
  
  # Check to see if the first column is all 1's.
  # If it isn't add the 1's for the intercept term
  if(!all(covariates[,1] == 1)){
    covariates <- as.matrix(cbind(1,covariates))
  }
  
  xb <- covariates %*% par
  sum(outcome * xb - xb - log(1 + exp(-xb)))
}

# Or the simplified version from Gary's slides:
#-sum(log(1 + exp((1-2*outcome) * xb)))

# Or
# sum(outcome * log(1 / (1+exp(-xb))) + (1-outcome) * log(1 - 1/(1+exp(-xb))))

# Or
# logit.xb <- inv.logit(covariates %*% par)
# sum(dbinom(outcome, 1, prob = logit.xb, log = T))




## Test that the function works:

logit.ll(par = rep(0, ncol(votes[,2:4]) + 1),
         covariates = votes[,2:4],
         outcome = votes$incwin) # -730.5771




## Estimate the MLE:

opt <- optim(par = rep(0, ncol(votes[,2:4]) + 1),
             fn = logit.ll,
             covariates = votes[,2:4],
             outcome = votes$incwin,
             control = list(fnscale = -1),
             hessian = T,
             method = "BFGS")

opt$par # Beta values for the intercept and 3 coefficients
sqrt(diag(-solve(opt$hessian))) # Standard errors


res <- cbind(opt$par, sqrt(diag(-solve(opt$hessian))))
colnames(res) <- c("Coefficient", "SE")
rownames(res) <- c("Intercept", "Open Seat", "Freshman", "Pres Vote")

#install.packages("stargazer")
require(stargazer)
stargazer(res, caption="DV: Incumbent Election Win")


#############################
## Out of sample predictions
#############################

#install.packages("mvtnorm")
require(mvtnorm)

sim.betas <- rmvnorm(n = 1,
                     mean = opt$par,
                     sigma = -solve(opt$hessian))
sim.betas






votes10 <- read.dta("votes10.dta")

votes10 <- as.matrix(cbind(1, votes10))

votes10[1,]

## We could get the mean of the predicted probabilities for
## this row by just multiplying by the
## beta coefficients and then turning that value back into
## a probability.

votes10[1,] %*% t(sim.betas)

1 / (1 + exp(-votes10[1,] %*% t(sim.betas)))
# or
require(boot)
sim.xb <- inv.logit(votes10[1,] %*% t(sim.betas)) 




## Now account for fundamental uncertainty by simulating from the stochastic function

# Take 1000 draws
draws <- rbinom(n = 10000, size = 1, prob = sim.xb)
mean(draws)




## But that doesn't fully account for the uncertainty in the estimation
## of the betas. We need to account for that uncertainty by assuming
## that beta is distributed multivariate normal and drawing 1000s of betas

sim.betas <- rmvnorm(n = 10000,
                     mean = opt$par,
                     sigma = -solve(opt$hessian))

head(sim.betas)
dim(sim.betas) ## 10,000 by 4 matrix of simulated betas



# Multiply the votes10 data by sim.betas to get 10,000 predicted probabilites
# for each of the 365 observations

pred.prob <- votes10[1,] %*% t(sim.betas) # transpose sim.betas so that you can do matrix multiplication

# But wait, these aren't probabilities!
hist(pred.prob)


# That's because they're still in terms of the logit of the probabilities.
# We need to untransform them using the inverse logit function
pred.prob <- c(inv.logit(pred.prob))



# Look at the results
hist(pred.prob,
     main = "Freshman incumbent where\n presidential candidate got 37%",
     xlab = "Predicted probability incumbent victory")

mean(pred.prob)
quantile(pred.prob, prob = c(.025,.975))
mean(pred.prob > .5) # The incumbent is predicted to win ~98% of the time in this district









################################################################
# We can look at the average number of seats won by the incumbents

# Draw our betas
sim.betas <- rmvnorm(n = 10000,
                     mean = opt$par,
                     sigma = -solve(opt$hessian))


# Multiply the votes10 data by sim.betas to get 10,000 predicted probabilites
# for each of the 365 observations

pred.prob <- inv.logit(votes10 %*% t(sim.betas))


# Now make it something useful:

results <- colSums(pred.prob > .5)

hist(results,
     main = "Projections of 2010 elections",
     xlab = "Number of seats won by incumbent's party\n (out of 365 contested races)",
     ylab = "Proportion",
     freq = F)





######################################################

## Using the GLM function to estimate logit models:

model.glm <- glm(incwin ~ open + freshman + incpres,
                 data = votes,
                 family = "binomial",
                 link = "logit")
summary(model.glm)

coef(model.glm)
opt$par

vcov(model.glm)
-solve(opt$hessian)




## Using Zelig to estimate logit models

#install.packages("Zelig")

require(Zelig)

model.zelig <- zelig(incwin ~ open + freshman + incpres,
                     data = votes,
                     model = "logit")

summary(model.zelig)
coef(model.zelig)
coef(model.glm) # exactly the same because zelig() wraps glm()



# Set the values of your covariates that you want zelig to use to 
# simulate your outcome:

values <- setx(model.zelig, open = 0, freshman = 1, incpres = 37)

# Simulate from your model, using the values you set above as the
# covariates

sims <- sim(model.zelig, values)
summary(sims)

plot(sims)